@@ -1,7 +1,7 @@
 /*
  *  package.c
  *
- *  Copyright (c) 2006-2016 Pacman Development Team <pacman-dev@archlinux.org>
+ *  Copyright (c) 2006-2018 Pacman Development Team <pacman-dev@archlinux.org>
  *  Copyright (c) 2002-2006 by Judd Vinet <jvinet@zeroflux.org>
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -92,7 +92,7 @@ static void make_aligned_titles(void)
 	size_t maxlen = 0;
 	int maxcol = 0;
 	static const wchar_t title_suffix[] = L" :";
-	wchar_t wbuf[ARRAYSIZE(titles)][TITLE_MAXLEN + ARRAYSIZE(title_suffix)];
+	wchar_t wbuf[ARRAYSIZE(titles)][TITLE_MAXLEN + ARRAYSIZE(title_suffix)] = {{ 0 }};
 	size_t wlen[ARRAYSIZE(wbuf)];
 	int wcol[ARRAYSIZE(wbuf)];
 	char *buf[ARRAYSIZE(wbuf)];
@@ -170,7 +170,7 @@ static void optdeplist_display(alpm_pkg_t *pkg, unsigned short cols)
 		alpm_depend_t *optdep = i->data;
 		char *depstring = alpm_dep_compute_string(optdep);
 		if(alpm_pkg_get_origin(pkg) == ALPM_PKG_FROM_LOCALDB) {
-			if(alpm_find_satisfier(alpm_db_get_pkgcache(localdb), optdep->name)) {
+			if(alpm_find_satisfier(alpm_db_get_pkgcache(localdb), depstring)) {
 				const char *installed = _(" [installed]");
 				depstring = realloc(depstring, strlen(depstring) + strlen(installed) + 1);
 				strcpy(depstring + strlen(depstring), installed);
@@ -230,7 +230,7 @@ void dump_pkg_full(alpm_pkg_t *pkg, int extra)
 			break;
 	}
 
-	alpm_pkgvalidation_t v = alpm_pkg_get_validation(pkg);
+	int v = alpm_pkg_get_validation(pkg);
 	if(v) {
 		if(v & ALPM_PKG_VALIDATION_NONE) {
 			validation = alpm_list_add(validation, _("None"));
@@ -568,5 +568,3 @@ int dump_pkg_search(alpm_db_t *db, alpm_list_t *targets, int show_status)
 
 	return 0;
 }
-
-/* vim: set noet: */
